# -*- coding: utf-8 -*-

from wtforms.fields import HiddenField, BooleanField


def add_helpers(app):
    """
    Create any Jinja2 helpers needed.
    """

    def is_hidden_field_filter(field):
        return isinstance(field, HiddenField)

    def is_boolean_field_filter(field):
        return isinstance(field, BooleanField)

    app.jinja_env.filters['is_hidden_field'] = is_hidden_field_filter
    app.jinja_env.filters['is_boolean_field'] = is_boolean_field_filter



# -*- coding: utf-8 -*-

import flask.ext.security as security
from flask.ext.admin import BaseView, expose, AdminIndexView
from flask.ext.admin.contrib.sqla import ModelView
from flask.ext.admin.base import MenuLink
from flask.ext.security.utils import encrypt_password
import flask


class AuthMixin(object):
    """
    Require user authentication to be accessible
    """

    def is_accessible(self):
        """
        the View is accessible if the User is authenticated
        """

        return security.current_user.is_authenticated()


class AdminMixin(object):
    """
    Require admin Role to be accessible
    """

    def is_accessible(self):
        """
        the View is accessible if the User has the Admin Role
        """

        return security.current_user.has_role("Admin")


class AuthView(AuthMixin, BaseView):
    """
    A View that requires authentication
    """
    pass


class AdminView(AdminMixin, BaseView):
    """
    A View that requires the Admin Role
    """
    pass


class AuthModelView(AuthMixin, ModelView):
    """
    A ModelView that requires authentication
    """
    pass


class AdminModelView(AdminMixin, ModelView):
    """
    A ModelView that requires the Admin Role
    """
    pass


class AuthenticatedMenuLink(AuthMixin, MenuLink):
    """
    A MenuLink that requires authentication
    """
    pass


class UserView(AdminModelView):
    """
    Manage the User Model
    """

    column_filters = ['email']
    column_exclude_list = ('password', 'active', 'confirmed_at')
    column_searchable_list = ('email', )
    can_delete = False

    create_template = 'admin/create_user.html'

    def create_model(self, form):
        self.model.register(
            email=form.data["email"],
            password=form.data["password"],
            confirmed=True,
            roles=["User"],
        )
        return flask.redirect(flask.url_for("user.index_view"))

    def update_model(self, form, model):
        original_password = model.password
        model.update(**form.data)
        if form.data["password"] != original_password:
            model.password = encrypt_password(form.data["password"])
            model.save()
        return flask.redirect(flask.url_for("user.index_view"))


class ForceLoginView(AdminIndexView):
    """
    Allocate the root URL and require authentication
    """

    def is_accessible(self):
        """
        the View is accessible if the User is authenticated
        """

        return security.current_user.is_authenticated()

    @expose('/')
    def index(self):
        return self.render("/admin/index.html")



# -*- coding: utf-8 -*-

from flask.ext.security import RoleMixin
#import flask.ext.security as security
from .. import db


class Role(db.Model, RoleMixin):
    """
    For the purpose of access controls, Roles can be used to create
    collections of users and give them permissions as a group.
    """

    id = db.Column(db.Integer(), primary_key=True)
    "integer -- primary key"

    name = db.Column(db.String(80), unique=True)
    "string -- what the role is called"

    description = db.Column(db.String(255))
    "string -- a sentence describing the role"

    def __str__(self):
        return self.name



import re
import os
from setuptools import setup


# mimicking flask-admin setup.py
# https://github.com/flask-admin/flask-admin/blob/master/setup.py
def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)


def read(fname):
    return open(fpath(fname)).read()


file_text = read(fpath('flask_diamond/__init__.py'))


def grep(attrname):
    pattern = r"{0}\W*=\W*'([^']+)'".format(attrname)
    strval, = re.findall(pattern, file_text)
    return strval


setup(
    version=grep('__version__'),
    name='Flask-Diamond',
    description="""\
        Flask-Diamond provides a path that can guide your thought and development.
        Flask-Diamond is the road that leads to other ideas.
        """,
    packages=[
        "flask_diamond",
        "flask_diamond.models",
        "flask_diamond.views",
        "flask_diamond.views.diamond",
        "flask_diamond.utils",
    ],
    scripts=[
        "bin/diamond-scaffold.sh",
    ],
    long_description=read('Readme.rst'),
    classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    include_package_data=True,
    keywords='',
    author=grep('__author__'),
    author_email=grep('__email__'),
    url='http://flask-diamond.readthedocs.org',
    install_requires=read('requirements.txt'),
    license='MIT',
    zip_safe=False,
)



# -*- coding: utf-8 -*-

from flask.ext.testing import TestCase
from flask import current_app
from flask_diamond import models


class GeneralTestCase(TestCase):
    def create_app(self):
        """
        Create a Flask-Diamond app for testing.
        """

        from flask_diamond import create_app, db
        self.app = create_app()
        self.app.config['TESTING'] = True
        self.app.config['WTF_CSRF_ENABLED'] = False
        self.db = db
        return self.app

    def setUp(self):
        """
        Prepare for a test case.
        """

        self.db.drop_all()
        self.db.create_all()
        models.User.add_system_users()
        current_app.logger.debug("setup complete")

    def tearDown(self):
        """
        Clean up after a test case.
        """

        self.db.session.remove()



# -*- coding: utf-8 -*-

import string
import random
import collections


# https://stackoverflow.com/questions/2257441/python-random-string-generation-with-upper-case-letters-and-digits
def id_generator(size=8, chars=None):
    """
    Create a random sequence of letters and numbers.

    :param size: the desired length of the sequence
    :type size: integer
    :param chars: the eligible character set to draw from when picking random characters
    :type chars: string
    :returns: a string with the random sequence
    """
    if chars is None:
        chars = string.ascii_uppercase + string.ascii_lowercase + string.digits
    return ''.join(random.choice(chars) for x in range(size))


# http://stackoverflow.com/questions/6027558/flatten-nested-python-dictionaries-compressing-keys
def flatten(d, parent_key=''):
    """
    Flatten nested python dictionaries by compressing keys

    :param d: the hierarchical dictionary to flatten
    :type d: dict
    :param parent_key: a prefix to apply to new keys (may be '')
    :type parent_key: string
    """

    items = []
    for k, v in d.items():
        new_key = parent_key + '.' + str(k) if parent_key else str(k)
        if isinstance(v, collections.MutableMapping):
            items.extend(flatten(v, new_key).items())
        elif isinstance(v, collections.MutableSequence):
            array_as_dict = dict(zip(range(0, len(v)), v))
            items.extend(flatten(array_as_dict, new_key).items())
        else:
            items.append((new_key, v))
    return dict(items)


def _u(obj, default=None):
    """
    Ensure an object is Unicode.

    :param obj: the object that needs to be unicode
    :type obj: str or unicode
    :returns: a unicode representation of the original object
    """

    if type(obj) == str:
        try:
            obj = unicode(obj)
        except UnicodeDecodeError:
            obj = unicode(obj.decode("latin1"))
        return obj
    elif type(obj) == unicode:
        return obj
    else:
        return default
